@@ -74,17 +74,18 @@ def uname_get_machine():
 
 def read_issue(filename="/etc/issue"):
     """
-    @return: list of strings in issue file, or None if issue file cannot be read/split
+    :returns: list of strings in issue file, or None if issue file cannot be read/split
     """
-    try:
-        if os.path.exists(filename):
-            with open(filename, 'r') as f:
-                return f.read().split()
-    except:
-        pass
+    if os.path.exists(filename):
+        with open(filename, 'r') as f:
+            return f.read().split()
     return None
 
-class OsNotDetected(Exception): pass
+class OsNotDetected(Exception):
+    """
+    Exception to indicate failure to detect operating system.
+    """
+    pass
 
 class OsDetector:
     """
@@ -92,34 +93,45 @@ class OsDetector:
     """
     def is_os(self):
         """
-        @return: if the specific OS which this class is designed to
-        detect is present.  Only one version of this class should
-        return for any version.
+        :returns: if the specific OS which this class is designed to
+          detect is present.  Only one version of this class should
+          return for any version.
         """
-        raise NotImplemented("is_os unimplemented")
+        raise NotImplementedError("is_os unimplemented")
 
     def get_version(self):
         """
-        @return: standardized version for this OS. (ala Ubuntu Hardy Heron = "8.04")
+        :returns: standardized version for this OS. (ala Ubuntu Hardy Heron = "8.04")
+        :raises: :exc:`OsNotDetected` if called on incorrect OS.
         """
-        raise NotImplemented("get_version unimplemented")
+        raise NotImplementedError("get_version unimplemented")
+
+    def get_codename(self):
+        """
+        :returns: codename for this OS. (ala Ubuntu Hardy Heron = "hardy").  If codenames are not available for this OS, return empty string.
+        :raises: :exc:`OsNotDetected` if called on incorrect OS.
+        """
+        raise NotImplementedError("get_codename unimplemented")
 
 class LsbDetect(OsDetector):
     """
-    Generic detector for Debian, Ubuntu, Mint, and Mandriva
+    Generic detector for Debian, Ubuntu, and Mint
     """
-    def __init__(self, lsb_name, get_version_fn):
+    def __init__(self, lsb_name, get_version_fn=None):
         self.lsb_name = lsb_name
-        self.get_version_fn = get_version_fn
 
     def is_os(self):
         return lsb_get_os() == self.lsb_name
 
     def get_version(self):
-        return self.get_version_fn()
+        if self.is_os():
+            return lsb_get_version()
+        raise OsNotDetected('called in incorrect OS')
         
-def mandriva_version():
-    return lsb_get_version()+uname_get_machine()
+    def get_codename(self):
+        if self.is_os():
+            return lsb_get_codename()
+        raise OsNotDetected('called in incorrect OS')
 
 class OpenSuse(OsDetector):
     """
@@ -133,16 +145,14 @@ class OpenSuse(OsDetector):
         return os_list and os_list[0] == "openSUSE"
 
     def get_version(self):
-        try:
-            if os.path.exists(self._brand_file):
-                with open(self._brand_file, 'r') as fh:
-                    os_list = fh.read().strip().split('\n')
-                    if len(os_list) == 2:
-                        os_list = os_list[1].split(' = ')
-                        if os_list[0] == "VERSION":
-                            return os_list[1]
-        except: pass
-        return False
+        if self.is_os() and os.path.exists(self._brand_file):
+            with open(self._brand_file, 'r') as fh:
+                os_list = fh.read().strip().split('\n')
+                if len(os_list) == 2:
+                    os_list = os_list[1].split(' = ')
+                    if os_list[0] == "VERSION":
+                        return os_list[1]
+        raise OsNotDetected('cannot get version on this OS')
 
 class Fedora(OsDetector):
     """
@@ -157,32 +167,53 @@ class Fedora(OsDetector):
         return os_list and os_list[0] == "Fedora" 
 
     def get_version(self):
-        os_list = read_issue(self._issue_file)
-        idx = os_list.index('release')
-        if idx > 0:
-            return os_list[idx+1]
-        else:
-            return False
+        if self.is_os():
+            os_list = read_issue(self._issue_file)
+            idx = os_list.index('release')
+            if idx > 0:
+                return os_list[idx+1]
+        raise OsNotDetected('cannot get version on this OS')
 
 class Rhel(Fedora):
     """
     Detect Redhat OS.
     """
-    def __init__(self, release_file="/etc/redhat-release", issue_file="/etc/issue"):
+    def __init__(self, release_file="/etc/redhat-release"):
         self._release_file = release_file
-        self._issue_file = issue_file
 
     def is_os(self):
         os_list = read_issue(self._release_file)
-        return os_list and os_list[2] == "Enterprise"
+        return os_list and os_list[:3] == ['Red', 'Hat', 'Enterprise']
 
     def get_version(self):
-        os_list = read_issue(self._issue_file)
-        if os_list[2] == "Enterprise":
-            return os_list[6]
-        else:
-            raise OsNotDetected("cannot determine RHEL version")
+        if self.is_os():
+            os_list = read_issue(self._release_file)
+            idx = os_list.index('release')
+            return os_list[idx+1]
+        raise OsNotDetected('called in incorrect OS')
 
+    def get_codename(self):
+        # taroon, nahant, tikanga, santiago, pensacola
+        if self.is_os():
+            os_list = read_issue(self._release_file)
+            idx = os_list.index('release')
+            matches = [x for x in os_list if x[0] == '(']
+            codename = matches[0][1:]
+            if codename[-1] == ')':
+                codename = codename[:-1]
+            return codename.lower()
+        raise OsNotDetected('called in incorrect OS')
+    
+# Source: http://en.wikipedia.org/wiki/Mac_OS_X#Versions
+_osx_codename_map = {4: 'tiger',
+                     5:  'leopard',
+                     6: 'snow',
+                     7: 'lion'}
+def _osx_codename(major, minor):
+    if major != 10 or minor not in _osx_codename_map:
+        raise OsNotDetected("unrecognized version: %s.%s"%(major, minor))
+    return _osx_codename_map[minor]
+    
 class OSX(OsDetector):
     """
     Detect OS X 
@@ -193,25 +224,21 @@ class OSX(OsDetector):
     def is_os(self):
         return os.path.exists(self._sw_vers_file)
     
+    def get_codename(self):
+        if self.is_os():
+            version = self.get_version()
+            import distutils.version # To parse version numbers
+            try:
+                ver = distutils.version.StrictVersion(version).version
+            except ValueError:
+                raise OsNotDetected("invalid version string: %s"%(version))
+            return _osx_codename(*ver[0:2])
+        raise OsNotDetected('called in incorrect OS')
+        
     def get_version(self):
-        import distutils.version # To parse version numbers
-        # REP 111 this should be the code name (e.g., lion, snow, tiger) #3570
-        std_out = _read_stdout([self._sw_vers_file,'-productVersion'])
-        ver = distutils.version.StrictVersion(std_out).version
-        if len(ver) < 2:
-            raise OsNotDetected("invalid version string: %s"%(std_out))
-        major, minor = ver[0:2]
-        # Source: http://en.wikipedia.org/wiki/Mac_OS_X#Versions
-        if major == 10 and minor == 4:
-            return 'tiger'
-        elif major == 10 and minor == 5:
-            return 'leopard'
-        elif major == 10 and minor == 6:
-            return 'snow'
-        elif major == 10 and minor == 7:
-            return 'lion'
-        else:
-            raise OsNotDetected("unrecognized version: %s"%(std_out))
+        if self.is_os():
+            return _read_stdout([self._sw_vers_file,'-productVersion']).strip()
+        raise OsNotDetected('called in incorrect OS')
 
 class Arch(OsDetector):
     """
@@ -224,7 +251,14 @@ class Arch(OsDetector):
         return os.path.exists(self._release_file)
 
     def get_version(self):
-        return ""
+        if self.is_os():
+            return ""
+        raise OsNotDetected('called in incorrect OS')        
+
+    def get_codename(self):
+        if self.is_os():
+            return ""
+        raise OsNotDetected('called in incorrect OS')        
 
 class Cygwin(OsDetector):
     """
@@ -234,7 +268,14 @@ class Cygwin(OsDetector):
         return os.path.exists("/usr/bin/cygwin1.dll")
     
     def get_version(self):
-        return _read_stdout(['uname','-r'])
+        if self.is_os():
+            return _read_stdout(['uname','-r'])
+        raise OsNotDetected('called in incorrect OS')        
+
+    def get_codename(self):
+        if self.is_os():
+            return ''
+        raise OsNotDetected('called in incorrect OS')        
 
 class Gentoo(OsDetector):
     """
@@ -248,11 +289,16 @@ class Gentoo(OsDetector):
         return os_list and os_list[0] == "Gentoo" and os_list[1] == "Base"
 
     def get_version(self):
-        os_list = read_issue(self._release_file)
-        if os_list[0] == "Gentoo" and os_list[1] == "Base":
-            return os_list[4]
-        else:
-            return False
+        if self.is_os():
+            os_list = read_issue(self._release_file)
+            if os_list[0] == "Gentoo" and os_list[1] == "Base":
+                return os_list[4]
+        raise OsNotDetected('called in incorrect OS')        
+
+    def get_codename(self):
+        if self.is_os():
+            return ''
+        raise OsNotDetected('called in incorrect OS')        
 
 class FreeBSD(OsDetector):
     """
@@ -263,19 +309,20 @@ class FreeBSD(OsDetector):
 
     def is_os(self):
         if os.path.exists(self._uname_file):
-            std_out = _read_stdout([filename])
+            std_out = _read_stdout([self._uname_file])
             return std_out.strip() == "FreeBSD"
         else:
             return False
 
     def get_version(self):
-        try:
-            if os.path.exists(self._uname_file):
-               return _read_stdout([self._uname_file, "-r"])
-            else:
-               return False
-        except: pass
-        return False
+        if self.is_os() and os.path.exists(self._uname_file):
+            return _read_stdout([self._uname_file, "-r"])
+        raise OsNotDetected('called in incorrect OS')        
+
+    def get_codename(self):
+        if self.is_os():
+            return ''
+        raise OsNotDetected('called in incorrect OS')        
 
 class OsDetect:
     """
@@ -291,16 +338,29 @@ class OsDetect:
         self._os_list = os_list
         self._os_name = None
         self._os_version = None
+        self._os_codename = None
+        self._os_detector = None
         self._override = False
 
     @staticmethod
     def register_default(os_name, os_detector):
-        OsDetect.default_os_list.append((os_name, os_detector))
+        """
+        Register detector to be used with all future instances of
+        :class:`OsDetect`.  The new detector will have precedence over
+        any previously registered detectors associated with *os_name*.
+
+        :param os_name: OS key associated with OS detector
+        :param os_detector: :class:`OsDetector` instance
+        """
+        OsDetect.default_os_list.insert(0, (os_name, os_detector))
         
     def detect_os(self):
         """
-        @return: (os_name, os_version)
-        @raise OsNotDetected: is OS could not be detected
+        Detect operating system.  Return value can be overridden by
+        the :env:`ROS_OS_OVERRIDE` environment variable.
+        
+        :returns: (os_name, os_version, os_codename), ``(str, str, str)``
+        :raises: :exc:`OsNotDetected` if OS could not be detected
         """
         if 'ROS_OS_OVERRIDE' in os.environ:
             self._os_name, self._os_version = os.environ["ROS_OS_OVERRIDE"].split(':')
@@ -310,28 +370,40 @@ class OsDetect:
                 if os_detector.is_os():
                     self._os_name = os_name
                     self._os_version = os_detector.get_version()
+                    self._os_codename = os_detector.get_codename()
+                    self._os_detector = os_detector
 
         if self._os_name:
-            return (self._os_name, self._os_version)            
-        else:
-            # No solution found
-            attempted_oss = [o.get_name() for o in self._os_list]
-            raise OsNotDetected("Could not detect OS, tried %s"%attempted_oss)
+            return self._os_name, self._os_version, self._os_codename
+        else: # No solution found
+            attempted = [x[0] for x in self._os_list]
+            raise OsNotDetected("Could not detect OS, tried %s"%attempted)
 
-    def get_detector(self, name):
+    def get_detector(self, name=None):
         """
-        Get detector used for specified OS name.
-        @raises: KeyError
-        """
-        try:
-            return [d for d_name, d in self._os_list if d_name == name][0]
-        except IndexError:
-            raise KeyError(name)
+        Get detector used for specified OS name, or the detector for this OS if name is ``None``.
         
-    def get_os(self):
-        if not self._os_detector:
-            self.detect_os()
-        return self._os_detector
+        :raises: :exc:`KeyError`
+        """
+        if name is None:
+            if not self._os_detector:
+                self.detect_os()
+            return self._os_detector
+        else:
+            try:
+                return [d for d_name, d in self._os_list if d_name == name][0]
+            except IndexError:
+                raise KeyError(name)
+        
+    def add_detector(self, name, detector):
+        """
+        Add detector to list of detectors used by this instance.  *detector* will override any previous
+        detectors associated with *name*.
+
+        :param name: OS name that detector matches
+        :param detector: :class:`OsDetector` instance
+        """
+        self._os_list.insert(0, (name, detector))
 
     def get_name(self):
         if not self._os_name:
@@ -340,32 +412,35 @@ class OsDetect:
 
     def get_version(self):
         if not self._os_version:
-            not self.detect_os()
+            self.detect_os()
         return self._os_version
 
+    def get_codename(self):
+        if not self._os_codename:
+            self.detect_os()
+        return self._os_codename
+
+OS_ARCH='arch'
+OS_CYGWIN='cygwin'
 OS_DEBIAN='debian'
-OS_UBUNTU='ubuntu'
+OS_FEDORA='fedora'
+OS_FREEBSD='freebsd'
+OS_GENTOO='gentoo'
 OS_MINT='mint'
-OS_MANDRIVA='mandriva'
+OS_OPENSUSE='opensuse'
 OS_OSX='osx'
-OS_ARCH='arch'
-OS_OPENSUSE='opensuse'
-OS_FEDORA='fedora'
 OS_RHEL='rhel'
-OS_GENTOO='gentoo'
-OS_CYGWIN='cygwin'
-OS_FREEBSD='freebsd'
+OS_UBUNTU='ubuntu'
 
-OsDetect.register_default(OS_DEBIAN, LsbDetect("Debian", lsb_get_codename))
-OsDetect.register_default(OS_UBUNTU, LsbDetect("Ubuntu", lsb_get_codename))
-OsDetect.register_default(OS_MINT, LsbDetect("Mint", lsb_get_version))
-OsDetect.register_default(OS_MANDRIVA, LsbDetect("MandrivaLinux", mandriva_version))
-OsDetect.register_default(OS_OSX, OSX())
 OsDetect.register_default(OS_ARCH, Arch())
-OsDetect.register_default(OS_OPENSUSE, OpenSuse())
-OsDetect.register_default(OS_FEDORA, Fedora())
-OsDetect.register_default(OS_RHEL, Rhel())
-OsDetect.register_default(OS_GENTOO, Gentoo())
 OsDetect.register_default(OS_CYGWIN, Cygwin())
+OsDetect.register_default(OS_DEBIAN, LsbDetect("Debian"))
+OsDetect.register_default(OS_FEDORA, Fedora())
 OsDetect.register_default(OS_FREEBSD, FreeBSD())
+OsDetect.register_default(OS_GENTOO, Gentoo())
+OsDetect.register_default(OS_MINT, LsbDetect("Mint"))
+OsDetect.register_default(OS_OPENSUSE, OpenSuse())
+OsDetect.register_default(OS_OSX, OSX())
+OsDetect.register_default(OS_RHEL, Rhel())
+OsDetect.register_default(OS_UBUNTU, LsbDetect("Ubuntu"))
     
